@@ -7,7 +7,6 @@ from xml.etree import ElementTree
 
 import pytest
 
-import salt.version
 from tests.support.virt import SaltVirtMinionContainerFactory
 
 docker = pytest.importorskip("docker")
@@ -20,19 +19,6 @@ pytestmark = [
 ]
 
 
-def _install_salt_dependencies(container):
-    dependencies = []
-    for package, version in salt.version.dependency_information():
-        if package not in ("packaging", "looseversion"):
-            # These are newer base dependencies which the container might not
-            # yet have
-            continue
-        dependencies.append(f"{package}=={version}")
-    if dependencies:
-        ret = container.run("python3", "-m", "pip", "install", *dependencies)
-        log.debug("Install missing dependecies ret: %s", ret)
-
-
 @pytest.fixture(scope="module")
 def virt_minion_0_id():
     return "virt-minion-0"
@@ -73,7 +59,6 @@ def virt_minion_0(
         skip_on_pull_failure=True,
         skip_if_docker_client_not_connectable=True,
     )
-    factory.before_start(_install_salt_dependencies, factory)
     factory.after_terminate(
         pytest.helpers.remove_stale_minion_key, salt_master, factory.id
     )
@@ -111,7 +96,6 @@ def virt_minion_1(
         skip_on_pull_failure=True,
         skip_if_docker_client_not_connectable=True,
     )
-    factory.before_start(_install_salt_dependencies, factory)
     factory.after_terminate(
         pytest.helpers.remove_stale_minion_key, salt_master, factory.id
     )
@@ -252,10 +236,15 @@ class TestVirtTest:
         assert len(caps["guests"]) >= 1
         assert caps["guests"][0]["os_type"] in ["hvm", "xen", "xenpvh", "exe"]
 
-    def test_cpu_baseline(self, salt_cli, virt_minion_0):
+    def test_cpu_baseline(self, salt_cli, virt_minion_0, grains):
         """
         Test virt.cpu_baseline
         """
+        if grains.get("osarch", "") != "x86_64":
+            raise pytest.skip.Exception(
+                f"Test is only meant to run on 'x86_64' architecture, not '{grains['osarch']}'",
+                _use_item_location=True,
+            )
         vendors = ["Intel", "ARM", "AMD"]
         ret = salt_cli.run(
             "virt.cpu_baseline", out="libvirt", minion_tgt=virt_minion_0.id
@@ -340,7 +329,12 @@ def virt_domain():
 
 
 @pytest.fixture
-def prep_virt(salt_cli, virt_minion_0, virt_minion_1, virt_domain):
+def prep_virt(salt_cli, virt_minion_0, virt_minion_1, virt_domain, grains):
+    if grains.get("osarch", "") != "x86_64":
+        raise pytest.skip.Exception(
+            f"Test is only meant to run on 'x86_64' architecture, not '{grains['osarch']}'",
+            _use_item_location=True,
+        )
     try:
         ret = salt_cli.run("virt.list_domains", minion_tgt=virt_minion_0.id)
         assert ret.returncode == 0, ret
@@ -382,13 +376,18 @@ def prep_virt(salt_cli, virt_minion_0, virt_minion_1, virt_domain):
 @pytest.mark.slow_test
 @pytest.mark.skip_if_binaries_missing("docker")
 class TestVirtMigrateTest:
-    def test_define_xml_path(self, salt_cli, virt_minion_0, virt_domain):
+    def test_define_xml_path(self, salt_cli, virt_minion_0, virt_domain, grains):
         """
         Define a new domain with virt.define_xml_path,
         verify that the new domain is shown with virt.list_domains,
         remove the domain with virt.undefine, and verifies that
         domain is no longer shown with virt.list_domains.
         """
+        if grains.get("osarch", "") != "x86_64":
+            raise pytest.skip.Exception(
+                f"Test is only meant to run on 'x86_64' architecture, not '{grains['osarch']}'",
+                _use_item_location=True,
+            )
         ret = salt_cli.run(
             "virt.define_xml_path",
             "/{}.xml".format(virt_domain),
@@ -418,11 +417,13 @@ class TestVirtMigrateTest:
         assert domains == []
 
     def test_ssh_migration(
-        self, salt_cli, virt_minion_0, virt_minion_1, prep_virt, virt_domain
+        self, salt_cli, virt_minion_0, virt_minion_1, prep_virt, virt_domain, grains
     ):
         """
         Test domain migration over SSH, TCP and TLS transport protocol
         """
+        if grains["os"] == "VMware Photon OS" and grains["osmajorrelease"] == 3:
+            pytest.skip("Skipping this test on PhotonOS 3")
         ret = salt_cli.run("virt.list_active_vms", minion_tgt=virt_minion_0.id)
         assert ret.returncode == 0, ret
 
